#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <iostream>
using namespace std;
#include "Commons.h"

//area graph structure
struct area {
	int id;
	double x;
	double y;
	double z;
};

//------------------------------------------------------------------------------
Area* importGraphAreaInstance(char *file) {
	Area *ar;
	ar = new Area[NUM_VERTEX];
	if (ar == NULL)
		return NULL;

	FILE *f = fopen(file, "r");

	fscanf(f, "id	x	y	z\n");

	for (int i = 0; i < NUM_VERTEX; i++) {
		fscanf(f, "%d %lf %lf %lf\n", &ar[i].id, &ar[i].x, &ar[i].y, &ar[i].z);
	}
	fclose(f);
	return ar;
}

int* importPPAInstance(char *file) {
	int *PPA;
	PPA = new int[NUM_VERTEX];

	if (PPA == NULL)
		return NULL;

	int i, id;
	double x, y, z;

	for (i = 0; i < NUM_VERTEX; i++) {
		PPA[i] = 0;
	}

	FILE *f = fopen(file, "r");

	fscanf(f, "id	x	y	z\n");

	for (i = 0; i < NUM_VERTEX_PPA; i++) {
		fscanf(f, "%d %lf %lf %lf\n", &id, &x, &y, &z);
		PPA[id - 1] = 1;
	}
	fclose(f);

	return PPA;
}

int* importInundInstance(char *file) {
	int *Inun;
	Inun = new int[NUM_VERTEX];

	if (Inun == NULL)
		return NULL;

	int i, id;
	double x, y, z;

	for (i = 0; i < NUM_VERTEX; i++) {
		Inun[i] = 0;
	}

	FILE *f = fopen(file, "r");

	fscanf(f, "id	x	y	z\n");

	for (i = 0; i < NUM_VERTEX_INUND; i++) {
		fscanf(f, "%d %lf %lf %lf\n", &id, &x, &y, &z);
		Inun[id - 1] = 1;
	}
	fclose(f);

	return Inun;
}

Slope* importSlopeInstance(char *file) {
	Slope *incl;
	incl = new Slope[NUM_VERTEX];
	if (incl == NULL)
		return NULL;

	FILE *f = fopen(file, "r");

	fscanf(f, "id	x	y	z	inclinacao\n");

	for (int i = 0; i < NUM_VERTEX; i++) {
		fscanf(f, "%d %lf %lf %lf %d\n", &incl[i].id, &incl[i].x, &incl[i].y, &incl[i].z, &incl[i].slopeValue);
	}
	fclose(f);
	return incl;
}

void importSourceRiverInstance(char *file, SourceRiver *sourceRiver, Turn *turn) {
#ifdef NUM_SOURCERIVER
	FILE *f = fopen(file, "r");

	fscanf(f, "id	x	y	z\n");

	for (int i = 0; i < NUM_SOURCERIVER; i++) {
		fscanf(f, "%d %lf %lf %lf\n", &sourceRiver[i].id, &sourceRiver[i].x, &sourceRiver[i].y, &sourceRiver[i].z);
		turn[i].id = sourceRiver[i].id;
		turn[i].x = sourceRiver[i].x;
		turn[i].y = sourceRiver[i].y;
		turn[i].z = sourceRiver[i].z;
	}
	fclose(f);
#endif
}
//------------------------------------------------------------------------------
void importTreesInstance(char *file, Tree *tree, Turn *turn) {
#ifdef NUM_SOURCERIVER
	FILE *f = fopen(file, "r");

	fscanf(f, "ID	UT	N_Arvore	DAP	HC	G	Volume	X	Y	Z\n");

	for (int i = 0; i < NUM_TREE; i++) {
		fscanf(f, "%d %d %d %lf %lf %lf %lf %lf %lf %lf\n", &tree[i].id, &tree[i].wu, &tree[i].number, &tree[i].DBH, &tree[i].H, &tree[i].basalArea, &tree[i].volume, &tree[i].x, &tree[i].y, &tree[i].z);
		turn[i + NUM_SOURCERIVER].id = tree[i].id;
		turn[i + NUM_SOURCERIVER].x = tree[i].x;
		turn[i + NUM_SOURCERIVER].y = tree[i].y;
		turn[i + NUM_SOURCERIVER].z = tree[i].z;
	}
	fclose(f);
#else
	FILE *f = fopen(file, "r");

	fscanf(f, "ID	UT	N_Arvore	DAP	HC	G	Volume	X	Y	Z\n");

	for (int i = 0; i < NUM_TREE; i++) {
		fscanf(f, "%d %d %d %lf %lf %lf %lf %lf %lf %lf\n", &tree[i].id, &tree[i].wu, &tree[i].number, &tree[i].DBH, &tree[i].H, &tree[i].basalArea, &tree[i].volume, &tree[i].x, &tree[i].y, &tree[i].z);
		turn[i].id = tree[i].id;
		turn[i].x = tree[i].x;
		turn[i].y = tree[i].y;
		turn[i].z = tree[i].z;
	}
	fclose(f);
#endif
}

//------------------------------------------------------------------------------
int RadiusTurnSearch(Area *ar, Turn *turn, int vertex) {
	double distance3d;
	int i;
	for (i = 0; i < NUM_TURN; i++) {
		distance3d = sqrt(pow((turn[i].x - ar[vertex].x), 2) + pow((turn[i].y - ar[vertex].y), 2) + pow((turn[i].z - ar[vertex].z), 2));

		if (distance3d <= AVOID_RADIUS) {
			return turn[i].id;
		}
	}
	return 0;
}

//------------------------------------------------------------------------------
int PPASearch(int *PPA, int vertex) {
	return PPA[vertex];
}

//------------------------------------------------------------------------------
int InundSearch(int *inund, int vertex) {
	return inund[vertex];
}

//------------------------------------------------------------------------------
int SlopeSearch(Slope *slope, int vertex) {
	//penalizing slope
	switch (slope[vertex].slopeValue) {
	case 1:
		return PENALTY_SLOPE_8;
	case 2:
		return PENALTY_SLOPE_10;
	case 3:
		return PENALTY_SLOPE_100;
	default:
		return PENALTY_SLOPE_8;
	}
}

//------------------------------------------------------------------------------
double CalculatesDistance(Area *ar, int source, int dest, int d3) {
	double distance;
	Area *vI = &ar[source];
	Area *vJ = &ar[dest];
	if (d3)
		distance = sqrt(pow((vI->x - vJ->x), 2) + pow((vI->y - vJ->y), 2) + pow((vI->z - vJ->z), 2));
	else
		distance = sqrt(pow((vI->x - vJ->x), 2) + pow((vI->y - vJ->y), 2));

	return distance;
}

//------------------------------------------------------------------------------
double CalculatesPenaltyDistance(Area *ar, int source, int dest, Turn *turn, int *PPA, int *inund, Slope *slope) {
	double distance;
	int indBarrier, indPPA, indInund;
	Area *vI = &ar[source];
	Area *vJ = &ar[dest];
	indBarrier = RadiusTurnSearch(ar, turn, dest);
	indPPA = PPASearch(PPA, dest);
	indInund = InundSearch(inund, dest);
	distance = sqrt(pow((vI->x - vJ->x), 2) + pow((vI->y - vJ->y), 2) + pow((vI->z - vJ->z), 2));

	if (indBarrier)
		distance *= PENALTY_BARRIER;

	if (indPPA)
		distance *= PENALTY_PPA;

	if (indInund)
		distance *= PENALTY_INUND;

	distance *= SlopeSearch(slope, dest);

	return distance;
}

//------------------------------------------------------------------------------

void SaveResultsConstraint(GroupRoadSolution *sol, char *fileResul, Area *ar, Turn *turn, int *PPA, int *inund, Slope *slope) {
	int i, current, npEcological = 0;
	int indBarrier, indPPA, indInund;
	float distance, pEcological = 0, pTotEcological = 0, pHydrographic = 0, pTotHydrographic = 0, pTopographical = 0, pTotTopographical = 0;
	
	int tree[NUM_TREE];
	for (i = 0; i < NUM_TREE; i++)
		tree[i] = 0;

	FILE *fResul = fopen(fileResul, "a+");
	fprintf(fResul, "-------------------RESULTADO DAS RESTRICOES----------------------\n");

	for (i = 0; i<NUM_ROAD; i++) {
		RoadSolution *estI = &sol->roads[i];
		current = estI->road.source;
		do {
			distance = estI->distance[estI->successor[current]] - estI->distance[current];
			if (estI->successor[current] != -1) {
				indBarrier = RadiusTurnSearch(ar, turn, estI->successor[current]);
				indPPA = PPASearch(PPA, estI->successor[current]);
				indInund = InundSearch(inund, estI->successor[current]);
				if (indBarrier) {
					pEcological = (distance * PENALTY_BARRIER) - distance;
					pTotEcological += pEcological;

					if (tree[indBarrier - 1] == 0) {
						tree[indBarrier - 1] = 1;
						npEcological++;
					}					
					distance += pEcological;
				}

				if (indPPA) {
					pHydrographic = (distance * PENALTY_PPA) - distance;
					pTotHydrographic += pHydrographic;
					distance += pHydrographic;
				}

				if (indInund) {
					pTopographical = (distance * PENALTY_INUND) - distance;
					distance += pTopographical;
				}
				pTopographical += (distance * SlopeSearch(slope, estI->successor[current])) - distance;
				pTotTopographical += pTopographical;
			}
			pEcological = 0;
			pHydrographic = 0;
			pTopographical = 0;
			current = estI->successor[current];
		} while (current != -1);

		fprintf(fResul, "Ext Total\t FO\t Num Arv\t Pen Eco\t Pen Hidr\t Pen Top\t Total\n");
		fprintf(fResul, "%.2f\t", estI->totalDistance);
		fprintf(fResul, "%.2f\t", estI->FO);
		fprintf(fResul, "%d\t", npEcological);
		fprintf(fResul, "%.2f\t", pTotEcological);
		fprintf(fResul, "%.2f\t", pTotHydrographic);
		fprintf(fResul, "%.2f\t", pTotTopographical);
		fprintf(fResul, "%.2f\n", pTotEcological + pTotHydrographic + pTotTopographical);
	}

	fclose(fResul);
}

void SaveResultsRoads(char *fileResul, char *fileSol, char *fileSummary, GroupRoadSolution *sol, int iter) {
	int i, j, iOrder;
	FILE *fResul = fopen(fileResul, "a+");
	FILE *fSol = fopen(fileSol, "a+");
	FILE *fSummary = fopen(fileSummary, "a+");
	fprintf(fResul, "-------------------RESULTADO----------------------\n");
	printf("-------------------RESULTADO----------------------\n");

	fprintf(fSol, "-------------------AGRUPAMENTO POR TRECHO----------------------\n");
	fprintf(fSol, "Dim iTrecho\n");

	for (i = 0; i<NUM_ROAD; i++) {
		fprintf(fResul, "Trecho: %d \n", i + 1);
		printf("Trecho: %d \n", i + 1);

		fprintf(fResul, "Vertice inicial: %d \n", sol->roads[i].road.source);
		printf("Vertice inicial: %d \n", sol->roads[i].road.source);

		fprintf(fResul, "Cod Vertice \tCod ant \tCod prox \tDistancia\n");
		printf("Cod Vertice \tCod ant \tCod prox \tDistancia\n");
		j = sol->roads[i].road.end;
		while (j != -1) {
			fprintf(fResul, "%d\t\t", j);
			printf("%d\t\t", j);
			fprintf(fResul, "%d\t\t", sol->roads[i].ancestor[j]);
			printf("%d\t\t", sol->roads[i].ancestor[j]);
			fprintf(fResul, "%d\t\t", sol->roads[i].successor[j]);
			printf("%d\t\t", sol->roads[i].successor[j]);
			fprintf(fResul, "%.2f\n", sol->roads[i].distance[j]);
			printf("%.2f\n", sol->roads[i].distance[j]);

			fprintf(fSol, "if [FID] = %d then iTrecho = %d end if\n", j, i + 1);

			j = sol->roads[i].ancestor[j];
		}
		fprintf(fResul, "Num vrtices rota: %d \n", sol->roads[i].routeNumVertex);
		printf("Num vrtices rota: %d \n", sol->roads[i].routeNumVertex);

		fprintf(fResul, "FO: %lf \n", sol->roads[i].FO);
		printf("FO: %lf \n", sol->roads[i].FO);

		fprintf(fResul, "Distancia total: %lf \n", sol->roads[i].totalDistance);
		printf("Distancia Total: %lf \n", sol->roads[i].totalDistance);

		printf("Tempo sol (em segundos): %lf \n", sol->roads[i].timeSol);
		fprintf(fResul, "Tempo sol (em segundos): %lf \n", sol->roads[i].timeSol);

		printf("Tempo (em segundos): %lf \n", sol->roads[i].time);
		fprintf(fResul, "Tempo (em segundos): %lf \n", sol->roads[i].time);

		fprintf(fResul, "\n");
		fprintf(fSol, "\n");
		printf("\n");
	}
	printf("Distancia total: %lf \n", sol->totalDistance);
	printf("FO total: %lf \n", sol->TotalFO);
	printf("Tempo total (em segundos): %.5f \n", sol->totalTime);

	if (iter == 1)
		fprintf(fSummary, "Iter\t Extensao \t FO total \t Tempo (seg)\n");
	fprintf(fSummary, "%d\t %lf \t %lf \t %.5f\n", iter, sol->totalDistance, sol->TotalFO, sol->totalTime);

	fclose(fSummary);
	fclose(fResul);

	fprintf(fSol, "\n-------------------ORDEM DO TRECHO----------------------\n");
	fprintf(fSol, "Dim iOrdem\n");

	for (i = 0; i < NUM_ROAD; i++) {
		iOrder = 1;
		j = sol->roads[i].road.end;
		while (j != -1) {
			fprintf(fSol, "if [FID] = %d then iOrdem = %d end if\n", j, iOrder);
			iOrder++;
			j = sol->roads[i].ancestor[j];
		}
		fprintf(fSol, "\n");
	}

	fclose(fSol);
}