#include "stdafx.h"
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <vector>
#include <utility>
#include <list>
#include <deque>
#include "Graph.h"
#include "TimeCounter.h"

using namespace std;

# define INF 0x3f3f3f3f

struct graph {
	int weighting;
	int numVertex;
	int maxNeighbor;
	int** edges;
	double** distances;
	double** weights;
	int* neighbor;
};

Graph* buildGraph(int numVertex, int maxNeighbor, int weighting) {
	Graph *gr = new Graph();
	if (gr != NULL) {
		int i;
		gr->numVertex   = numVertex;
		gr->maxNeighbor = maxNeighbor;
		gr->weighting   = (weighting != 0) ? 1 : 0;
		gr->neighbor    = new int[numVertex];

		for (i = 0; i < numVertex; i++)
			gr->neighbor[i] = 0;
		//edge matrix
		gr->edges = new int *[numVertex];
		if (gr->edges == NULL)
			return NULL;
		for (i = 0; i < numVertex; i++) {
			gr->edges[i] = new int[maxNeighbor];
			if (gr->edges[i] == NULL)
				return NULL;
		}
		//weight matrix
		if (gr->weighting) {
			//penalty
			gr->distances = new double *[numVertex];
			if (gr->distances == NULL)
				return NULL;
			for (i = 0; i < numVertex; i++) {
				gr->distances[i] = new double[maxNeighbor];
				if (gr->distances[i] == NULL)
					return NULL;
			}
			gr->weights = new double *[numVertex];
			if (gr->weights == NULL)
				return NULL;
			for (i = 0; i < numVertex; i++) {
				gr->weights[i] = new double[maxNeighbor];
				if (gr->weights[i] == NULL)
					return NULL;
			}
		}
	}
	return gr;
}

void freeGraph(Graph* gr) {
	if (gr != NULL) {
		int i;
		//releases the edge matrix
		for (i = 0; i < gr->numVertex; i++)
			delete[]gr->edges[i];
		delete[]gr->edges;

		//releases the weight matrix
		if (gr->weighting) {
			for (i = 0; i<gr->numVertex; i++)
				delete[]gr->distances[i];
			delete[]gr->distances;

			for (i = 0; i<gr->numVertex; i++)
				delete[]gr->weights[i];
			delete[]gr->weights;
		}
		delete[]gr->neighbor;
		delete[]gr;
	}
}

int insertEdge(Graph* gr, int source, int dest, double distance, double weight) {
	if (gr == NULL)
		return 0;

	//check if vertex exists
	if (source < 0 || source >= gr->numVertex)
		return 0;
	if (dest < 0 || dest >= gr->numVertex)
		return 0;

	gr->edges[source][gr->neighbor[source]] = dest;
	if (gr->weighting) {
		gr->distances[source][gr->neighbor[source]] = distance;
		gr->weights[source][gr->neighbor[source]] = weight;
	}
	gr->neighbor[source]++;

	return 1;
}

int shortestDistanceSearch(double *weight, int *visited, int numVertex) {
	int i, smaller = -1, first = 1;
	for (i = 0; i < numVertex; i++) {
		//seeks vertex with less distance that has not been visited
		if (weight[i] >= 0 && visited[i] == 0) {
			if (first) {
				smaller = i;
				first   = 0;
			}
			else {
				if (weight[smaller] > weight[i])
					smaller = i;
			}
		}
	}
	return smaller;
}

RoadSolution Dijkstra(Graph *gr, int source, int dest) {
	double tStart = get_time();
	RoadSolution sol;
	sol.negativeCycle = 0;
	sol.road.source = source;
	sol.road.end = dest;
	//ancestor: stores the previous vertex of the current vertex
	//distance: stores the distance from the current vertex to the starting vertex
	int i, cont, numVertex, ind, *visited, u;
	cont = numVertex = gr->numVertex;
	//creates auxiliary vector and initializes distances and anteriors
	visited = new int[numVertex];
	for (i = 0; i < numVertex; i++) {
		sol.ancestor[i]   = -1;
		sol.successor[i]  = -1;
		sol.distance[i]   = -1;
		sol.weight[i]     = -1;
		visited[i]        = 0;
	}

	sol.distance[sol.road.source] = 0;
	sol.weight[sol.road.source] = 0;
	while (cont > 0) {
		//vertex search with shorter distance and mark as visited
		u = shortestDistanceSearch(sol.weight, visited, numVertex);
		if (u == -1)
			break;

		visited[u] = 1;
		cont--;
		for (i = 0; i < gr->neighbor[u]; i++) {
			//for each neighbor vertex
			ind = gr->edges[u][i];
			//update distances from neighbors
			if (sol.weight[ind] < 0) {
				sol.distance[ind] = sol.distance[u] + gr->distances[u][i];
				//or the weight of the edge
				sol.weight[ind] = sol.weight[u] + gr->weights[u][i];
				sol.ancestor[ind] = u;
				sol.successor[u] = ind;
			}
			else {
				if ((sol.weight[ind]) > (sol.weight[u] + gr->weights[u][i])) {
					sol.distance[ind] = sol.distance[u] + gr->distances[u][i];
					//or the weight of the edge
					sol.weight[ind] = sol.weight[u] + gr->weights[u][i];
					sol.ancestor[ind] = u;
					sol.successor[u] = ind;
				}
			}
		}
	}
	delete[]visited;
	//counting the vertices of the route
	sol.routeNumVertex = 0;
	i = sol.road.end;
	while (i != -1) {
		sol.successor[sol.ancestor[i]] = i;
		i = sol.ancestor[i];
		sol.routeNumVertex++;
	}
	sol.FO = sol.weight[sol.road.end];
	sol.totalDistance = sol.distance[sol.road.end];
	sol.time = (get_time() - tStart) / 100.0;
	sol.timeSol = sol.time;

	return sol;
}

RoadSolution DijkstraOD(Graph *gr, int source, int dest) {
	double tStart = get_time();
	RoadSolution sol;
	sol.negativeCycle = 0;
	sol.road.source = source;
	sol.road.end = dest;
	//ancestor: stores the previous vertex of the current vertex
	//distance: stores the distance from the current vertex to the starting vertex
	int i, current, numVertex, ind, *visited, u;
	numVertex = gr->numVertex;
	current = sol.road.source;
	//creates auxiliary vector and initializes distances and anteriors
	visited = new int[numVertex];
	for (i = 0; i < numVertex; i++) {
		sol.ancestor[i]   = -1;
		sol.successor[i]  = -1;
		sol.distance[i]   = -1;
		sol.weight[i]     = -1;
		visited[i]        = 0;
	}

	sol.distance[sol.road.source] = 0;
	sol.weight[sol.road.source] = 0;
	while (current != sol.road.end) {
		//vertex search with shorter distance and mark as visited
		u = shortestDistanceSearch(sol.weight, visited, numVertex);
		if (u == -1)
			break;

		visited[u] = 1;
		for (i = 0; i < gr->neighbor[u]; i++) {
			//for each neighbor vertex
			ind = gr->edges[u][i];
			//update distances from neighbors
			if (sol.weight[ind] < 0) {
				sol.distance[ind] = sol.distance[u] + gr->distances[u][i];
				//or the weight of the edge
				sol.weight[ind] = sol.weight[u] + gr->weights[u][i];
				sol.ancestor[ind] = u;
			}
			else {
				if ((sol.weight[ind]) > (sol.weight[u] + gr->weights[u][i])) {
					sol.distance[ind] = sol.distance[u] + gr->distances[u][i];
					//or the weight of the edge
					sol.weight[ind] = sol.weight[u] + gr->weights[u][i];
					sol.ancestor[ind] = u;
				}
			}
		}
		current = u;
	}
	delete[]visited;

	//counting the vertices of the route
	sol.routeNumVertex = 0;
	i = sol.road.end;
	while (i != -1) {
		sol.successor[sol.ancestor[i]] = i;
		i = sol.ancestor[i];
		sol.routeNumVertex++;
	}
	sol.FO = sol.weight[sol.road.end];
	sol.totalDistance = sol.distance[sol.road.end];
	sol.time = (get_time() - tStart) / 100.0;
	sol.timeSol = sol.time;
	return sol;
}

RoadSolution DijkstraDial(Graph *gr, int source, int dest) {
	double tStart = get_time();
	RoadSolution sol;
	sol.negativeCycle = 0;
	sol.road.source = source;
	sol.road.end = dest;

	int i, numVertex, ind, u;
	numVertex = gr->numVertex;

	vector<pair<int, list<int>::iterator>> dist(numVertex);
	//create the buckets
	list<int> bucket[NUM_SIZE_BUCKET];
	for (i = 0; i < numVertex; i++) {
		sol.ancestor[i]   = -1;
		sol.successor[i]  = -1;
		sol.distance[i]   = INF;
		dist[i].first     = INF;
		sol.weight[i]     = INF;
	}

	sol.distance[sol.road.source] = 0;
	sol.weight[sol.road.source] = 0;
	//initializing dial variables
	bucket[0].push_back(sol.road.source);
	dist[sol.road.source].first = 0;
	int idx = 0;
	while (1) {
		// Pass through the buckets sequentially to a non-empty
		// bucket found
		while (bucket[idx].size() == 0 && idx < NUM_SIZE_BUCKET-1)
			idx++;

		// If all buckets are empty, concluded the search
		if (idx == NUM_SIZE_BUCKET-1)
			break;

		// Take the first corner of the bucket and remove it
		int u = bucket[idx].front(); //returns the reference to the first element of a vector
		bucket[idx].pop_front(); //removes the first element of a list

		// Process all adjacent vertex 'u' and
		// update the distance if necessary.
		for (i = 0; i < gr->neighbor[u]; i++) {
			//para cada vertice vizinho
			ind = gr->edges[u][i];

			//update distances from neighbors
			if (sol.weight[ind] >(sol.weight[u] + gr->weights[u][i])) {
				
				//if the weight other than inf then there is no minor path so far
				if (sol.weight[ind] != INF) {
					//then delete
					bucket[(int)(sol.weight[ind] / 10)].erase(dist[ind].second);
				}

				//updates the distance
				dist[ind].first = (int)((sol.weight[u] + gr->weights[u][i]) / 10);

				sol.distance[ind] = sol.distance[u] + gr->distances[u][i];
				//or edge weight
				sol.weight[ind] = sol.weight[u] + gr->weights[u][i];
				sol.ancestor[ind] = u;
				sol.successor[u] = ind;

				//pushing the ind vertex to the updated distance bucket
				bucket[dist[ind].first].push_front(ind);
				// storing the updated iterator in dist [ind].second
				dist[ind].second = bucket[dist[ind].first].begin();
			}
		}
	}
	//counting the vertices of the route
	sol.routeNumVertex = 0;
	i = sol.road.end;
	while (i != -1) {
		sol.successor[sol.ancestor[i]] = i;
		i = sol.ancestor[i];
		sol.routeNumVertex++;
	}
	sol.FO = sol.weight[sol.road.end];
	sol.totalDistance = sol.distance[sol.road.end];
	sol.time = (get_time() - tStart) / 100.0;
	sol.timeSol = sol.time;

	return sol;
}

RoadSolution bellmanford(Graph *gr, int source, int dest) {
	double tStart = get_time();
	RoadSolution sol;
	sol.negativeCycle = 0;
	sol.road.source = source;
	sol.road.end = dest;

	int i, j, numVertex, ind, u;
	numVertex = gr->numVertex;

	for (i = 0; i < numVertex; i++) {
		sol.ancestor[i] = -1;
		sol.successor[i] = -1;
		sol.distance[i] = INFINITY;
		sol.weight[i] = INFINITY;
	}

	sol.distance[sol.road.source] = 0;
	sol.weight[sol.road.source] = 0;

	for (i = 1; i <= numVertex - 1; i++) {
		for (u = 0; u < numVertex; u++) {
			for (j = 0; j < gr->neighbor[u]; j++) {
				//for each neighbor vertex
				ind = gr->edges[u][j];
				//update distances from neighbors

				if (sol.weight[u] != INFINITY && (sol.weight[ind]) >(sol.weight[u] + gr->weights[u][j])) {
					sol.distance[ind] = sol.distance[u] + gr->distances[u][j];
					//or edge weight
					sol.weight[ind] = sol.weight[u] + gr->weights[u][j];
					sol.ancestor[ind] = u;
					sol.successor[u] = ind;
				}
			}
		}
	}

	for (u = 0; u < numVertex; u++) {
		for (j = 0; j < gr->neighbor[u]; j++) {
			//for each neighbor vertex
			ind = gr->edges[u][j];

			if (sol.weight[u] != INFINITY && (sol.weight[ind]) >(sol.weight[u] + gr->weights[u][j])) {
				sol.negativeCycle = 1;
				return sol;
			}
		}
	}
	
	//counting the vertices of the route
	sol.routeNumVertex = 0;
	i = sol.road.end;
	while (i != -1) {
		sol.successor[sol.ancestor[i]] = i;
		i = sol.ancestor[i];
		sol.routeNumVertex++;
	}
	sol.FO = sol.weight[sol.road.end];
	sol.totalDistance = sol.distance[sol.road.end];
	sol.time = (get_time() - tStart) / 100.0;
	sol.timeSol = sol.time;

	return sol;
}

RoadSolution DEsopoPape(Graph *gr, int source, int dest) {
	double tStart = get_time();
	RoadSolution sol;
	sol.negativeCycle = 0;
	sol.road.source = source;
	sol.road.end = dest;
	int i, numVertex, ind, u;
	numVertex = gr->numVertex;
	vector<int> m(numVertex, 2);
	deque<int> q;
	
	for (i = 0; i < numVertex; i++) {
		sol.ancestor[i] = -1;
		sol.successor[i] = -1;
		sol.distance[i] = INF;
		sol.weight[i] = INF;
	}

	sol.distance[sol.road.source] = 0;
	sol.weight[sol.road.source] = 0;
	//initializing desopo-pape variables
	q.push_back(sol.road.source);
	while (!q.empty()) {
		u = q.front();
		q.pop_front();
		m[u] = 0;

		// Process all adjacent vertex 'u' and
		// update the distance if necessary.
		for (i = 0; i < gr->neighbor[u]; i++) {
			//for each neighbor vertex
			ind = gr->edges[u][i];

			//update distances from neighbors
			if (sol.weight[ind] >(sol.weight[u] + gr->weights[u][i])) {
				// updates the distance
				sol.distance[ind] = sol.distance[u] + gr->distances[u][i];
				//or edge weight
				sol.weight[ind] = sol.weight[u] + gr->weights[u][i];
				sol.ancestor[ind] = u;
				sol.successor[u] = ind;

				if (m[ind] == 2) {
					m[ind] = 1;
					q.push_back(ind);
				}
				else if (m[ind] == 0) {
					m[ind] = 1;
					q.push_front(ind);
				}
			}
		}
	}
	//counting the vertices of the route
	sol.routeNumVertex = 0;
	i = sol.road.end;
	while (i != -1) {
		sol.successor[sol.ancestor[i]] = i;
		i = sol.ancestor[i];
		sol.routeNumVertex++;
	}
	sol.FO = sol.weight[sol.road.end];
	sol.totalDistance = sol.distance[sol.road.end];
	sol.time = (get_time() - tStart) / 100.0;
	sol.timeSol = sol.time;

	return sol;
}

void insertAreaEdge(Graph *gr, Area *ar, Turn *turn, int *PPA, int *inund, Slope *slope) {
	int i, j, cont=0;
	double distance, distance3d, penalizedDistance;
	for (i = 0; i < NUM_VERTEX; i++) {
		for (j = 0; j < NUM_VERTEX; j++) {
			distance = CalculatesDistance(ar, i, j, 0);

			if (distance > 0 && distance <= NUM_DIST_MAX_NEIGHBOR) {
				distance3d = CalculatesDistance(ar, i, j, 1);
				penalizedDistance = CalculatesPenaltyDistance(ar, i, j, turn, PPA, inund, slope);

				//insert edge
				insertEdge(gr, i, j, distance3d, penalizedDistance);
				cont++;
			}
		}
	}
}

RoadSolution FillsRoadCompany(int road, int size, Graph *gr, Area *ar, Turn *turn, int *PPA, int *inund, Slope *slope) {
	int i;
	int *current = new int[size];
	int *successor = new int[size];
	RoadSolution sol;
	sol.negativeCycle = 0;

	for (i = 0; i < NUM_VERTEX; i++) {
		sol.ancestor[i] = -1;
		sol.successor[i] = -1;
		sol.distance[i] = -1;
		sol.weight[i] = -1;
	}

	switch (road)
	{
	case 1:
		sol.road.source = 27;
		sol.road.end = 1802;

		current[0] = 27; successor[0] = 80;
		current[1] = 80; successor[1] = 79;
		current[2] = 79; successor[2] = 132;
		current[3] = 132; successor[3] = 131;
		current[4] = 131; successor[4] = 184;
		current[5] = 184; successor[5] = 183;
		current[6] = 183; successor[6] = 236;
		current[7] = 236; successor[7] = 290;
		current[8] = 290; successor[8] = 344;
		current[9] = 344; successor[9] = 398;
		current[10] = 398; successor[10] = 452;
		current[11] = 452; successor[11] = 506;
		current[12] = 506; successor[12] = 560;
		current[13] = 560; successor[13] = 614;
		current[14] = 614; successor[14] = 668;
		current[15] = 668; successor[15] = 722;
		current[16] = 722; successor[16] = 776;
		current[17] = 776; successor[17] = 830;
		current[18] = 830; successor[18] = 884;
		current[19] = 884; successor[19] = 938;
		current[20] = 938; successor[20] = 992;
		current[21] = 992; successor[21] = 1046;
		current[22] = 1046; successor[22] = 1100;
		current[23] = 1100; successor[23] = 1154;
		current[24] = 1154; successor[24] = 1208;
		current[25] = 1208; successor[25] = 1262;
		current[26] = 1262; successor[26] = 1316;
		current[27] = 1316; successor[27] = 1370;
		current[28] = 1370; successor[28] = 1424;
		current[29] = 1424; successor[29] = 1478;
		current[30] = 1478; successor[30] = 1532;
		current[31] = 1532; successor[31] = 1586;
		current[32] = 1586; successor[32] = 1640;
		current[33] = 1640; successor[33] = 1694;
		current[34] = 1694; successor[34] = 1748;
		current[35] = 1748; successor[35] = 1802;
		break;
	case 2:
		sol.road.source = 82;
		sol.road.end = 15901;

		current[0] = 82; successor[0] = 241;
		current[1] = 241; successor[1] = 400;
		current[2] = 400; successor[2] = 399;
		current[3] = 399; successor[3] = 558;
		current[4] = 558; successor[4] = 557;
		current[5] = 557; successor[5] = 716;
		current[6] = 716; successor[6] = 715;
		current[7] = 715; successor[7] = 874;
		current[8] = 874; successor[8] = 873;
		current[9] = 873; successor[9] = 1032;
		current[10] = 1032; successor[10] = 1031;
		current[11] = 1031; successor[11] = 1190;
		current[12] = 1190; successor[12] = 1189;
		current[13] = 1189; successor[13] = 1348;
		current[14] = 1348; successor[14] = 1347;
		current[15] = 1347; successor[15] = 1506;
		current[16] = 1506; successor[16] = 1505;
		current[17] = 1505; successor[17] = 1664;
		current[18] = 1664; successor[18] = 1663;
		current[19] = 1663; successor[19] = 1822;
		current[20] = 1822; successor[20] = 1821;
		current[21] = 1821; successor[21] = 1981;
		current[22] = 1981; successor[22] = 2141;
		current[23] = 2141; successor[23] = 2301;
		current[24] = 2301; successor[24] = 2461;
		current[25] = 2461; successor[25] = 2621;
		current[26] = 2621; successor[26] = 2781;
		current[27] = 2781; successor[27] = 2941;
		current[28] = 2941; successor[28] = 3101;
		current[29] = 3101; successor[29] = 3261;
		current[30] = 3261; successor[30] = 3421;
		current[31] = 3421; successor[31] = 3581;
		current[32] = 3581; successor[32] = 3741;
		current[33] = 3741; successor[33] = 3901;
		current[34] = 3901; successor[34] = 4061;
		current[35] = 4061; successor[35] = 4221;
		current[36] = 4221; successor[36] = 4381;
		current[37] = 4381; successor[37] = 4541;
		current[38] = 4541; successor[38] = 4701;
		current[39] = 4701; successor[39] = 4861;
		current[40] = 4861; successor[40] = 5021;
		current[41] = 5021; successor[41] = 5181;
		current[42] = 5181; successor[42] = 5341;
		current[43] = 5341; successor[43] = 5501;
		current[44] = 5501; successor[44] = 5661;
		current[45] = 5661; successor[45] = 5821;
		current[46] = 5821; successor[46] = 5981;
		current[47] = 5981; successor[47] = 6141;
		current[48] = 6141; successor[48] = 6301;
		current[49] = 6301; successor[49] = 6461;
		current[50] = 6461; successor[50] = 6621;
		current[51] = 6621; successor[51] = 6781;
		current[52] = 6781; successor[52] = 6941;
		current[53] = 6941; successor[53] = 7101;
		current[54] = 7101; successor[54] = 7261;
		current[55] = 7261; successor[55] = 7421;
		current[56] = 7421; successor[56] = 7581;
		current[57] = 7581; successor[57] = 7741;
		current[58] = 7741; successor[58] = 7901;
		current[59] = 7901; successor[59] = 8061;
		current[60] = 8061; successor[60] = 8221;
		current[61] = 8221; successor[61] = 8381;
		current[62] = 8381; successor[62] = 8541;
		current[63] = 8541; successor[63] = 8701;
		current[64] = 8701; successor[64] = 8861;
		current[65] = 8861; successor[65] = 9021;
		current[66] = 9021; successor[66] = 9181;
		current[67] = 9181; successor[67] = 9341;
		current[68] = 9341; successor[68] = 9501;
		current[69] = 9501; successor[69] = 9661;
		current[70] = 9661; successor[70] = 9821;
		current[71] = 9821; successor[71] = 9981;
		current[72] = 9981; successor[72] = 10141;
		current[73] = 10141; successor[73] = 10301;
		current[74] = 10301; successor[74] = 10461;
		current[75] = 10461; successor[75] = 10621;
		current[76] = 10621; successor[76] = 10781;
		current[77] = 10781; successor[77] = 10941;
		current[78] = 10941; successor[78] = 11101;
		current[79] = 11101; successor[79] = 11261;
		current[80] = 11261; successor[80] = 11421;
		current[81] = 11421; successor[81] = 11581;
		current[82] = 11581; successor[82] = 11741;
		current[83] = 11741; successor[83] = 11901;
		current[84] = 11901; successor[84] = 12061;
		current[85] = 12061; successor[85] = 12221;
		current[86] = 12221; successor[86] = 12381;
		current[87] = 12381; successor[87] = 12541;
		current[88] = 12541; successor[88] = 12701;
		current[89] = 12701; successor[89] = 12861;
		current[90] = 12861; successor[90] = 13021;
		current[91] = 13021; successor[91] = 13181;
		current[92] = 13181; successor[92] = 13341;
		current[93] = 13341; successor[93] = 13501;
		current[94] = 13501; successor[94] = 13661;
		current[95] = 13661; successor[95] = 13821;
		current[96] = 13821; successor[96] = 13981;
		current[97] = 13981; successor[97] = 14141;
		current[98] = 14141; successor[98] = 14301;
		current[99] = 14301; successor[99] = 14461;
		current[100] = 14461; successor[100] = 14621;
		current[101] = 14621; successor[101] = 14781;
		current[102] = 14781; successor[102] = 14941;
		current[103] = 14941; successor[103] = 15101;
		current[104] = 15101; successor[104] = 15261;
		current[105] = 15261; successor[105] = 15421;
		current[106] = 15421; successor[106] = 15581;
		current[107] = 15581; successor[107] = 15741;
		current[108] = 15741; successor[108] = 15901;
		break;
	case 3:
		sol.road.source = 27;
		sol.road.end = 3683;

		current[0] = 27; successor[0] = 80;
		current[1] = 80; successor[1] = 79;
		current[2] = 79; successor[2] = 132;
		current[3] = 132; successor[3] = 131;
		current[4] = 131; successor[4] = 184;
		current[5] = 184; successor[5] = 183;
		current[6] = 183; successor[6] = 236;
		current[7] = 236; successor[7] = 290;
		current[8] = 290; successor[8] = 344;
		current[9] = 344; successor[9] = 398;
		current[10] = 398; successor[10] = 452;
		current[11] = 452; successor[11] = 506;
		current[12] = 506; successor[12] = 560;
		current[13] = 560; successor[13] = 614;
		current[14] = 614; successor[14] = 668;
		current[15] = 668; successor[15] = 722;
		current[16] = 722; successor[16] = 776;
		current[17] = 776; successor[17] = 830;
		current[18] = 830; successor[18] = 884;
		current[19] = 884; successor[19] = 938;
		current[20] = 938; successor[20] = 992;
		current[21] = 992; successor[21] = 1046;
		current[22] = 1046; successor[22] = 1100;
		current[23] = 1100; successor[23] = 1154;
		current[24] = 1154; successor[24] = 1208;
		current[25] = 1208; successor[25] = 1262;
		current[26] = 1262; successor[26] = 1316;
		current[27] = 1316; successor[27] = 1370;
		current[28] = 1370; successor[28] = 1424;
		current[29] = 1424; successor[29] = 1478;
		current[30] = 1478; successor[30] = 1532;
		current[31] = 1532; successor[31] = 1586;
		current[32] = 1586; successor[32] = 1640;
		current[33] = 1640; successor[33] = 1694;
		current[34] = 1694; successor[34] = 1748;
		current[35] = 1748; successor[35] = 1802;
		current[36] = 1802; successor[36] = 1801;
		current[37] = 1801; successor[37] = 1800;
		current[38] = 1800; successor[38] = 1799;
		current[39] = 1799; successor[39] = 1798;
		current[40] = 1798; successor[40] = 1797;
		current[41] = 1797; successor[41] = 1796;
		current[42] = 1796; successor[42] = 1795;
		current[43] = 1795; successor[43] = 1794;
		current[44] = 1794; successor[44] = 1793;
		current[45] = 1793; successor[45] = 1847;
		current[46] = 1847; successor[46] = 1901;
		current[47] = 1901; successor[47] = 1955;
		current[48] = 1955; successor[48] = 2009;
		current[49] = 2009; successor[49] = 2063;
		current[50] = 2063; successor[50] = 2117;
		current[51] = 2117; successor[51] = 2171;
		current[52] = 2171; successor[52] = 2225;
		current[53] = 2225; successor[53] = 2279;
		current[54] = 2279; successor[54] = 2333;
		current[55] = 2333; successor[55] = 2387;
		current[56] = 2387; successor[56] = 2441;
		current[57] = 2441; successor[57] = 2495;
		current[58] = 2495; successor[58] = 2549;
		current[59] = 2549; successor[59] = 2603;
		current[60] = 2603; successor[60] = 2657;
		current[61] = 2657; successor[61] = 2711;
		current[62] = 2711; successor[62] = 2765;
		current[63] = 2765; successor[63] = 2819;
		current[64] = 2819; successor[64] = 2873;
		current[65] = 2873; successor[65] = 2927;
		current[66] = 2927; successor[66] = 2981;
		current[67] = 2981; successor[67] = 3035;
		current[68] = 3035; successor[68] = 3089;
		current[69] = 3089; successor[69] = 3143;
		current[70] = 3143; successor[70] = 3197;
		current[71] = 3197; successor[71] = 3251;
		current[72] = 3251; successor[72] = 3305;
		current[73] = 3305; successor[73] = 3359;
		current[74] = 3359; successor[74] = 3413;
		current[75] = 3413; successor[75] = 3467;
		current[76] = 3467; successor[76] = 3521;
		current[77] = 3521; successor[77] = 3575;
		current[78] = 3575; successor[78] = 3629;
		current[79] = 3629; successor[79] = 3683;
		break;
	case 4:
		sol.road.source = 82;
		sol.road.end = 32674;

		current[0] = 82; successor[0] = 241;
		current[1] = 241; successor[1] = 400;
		current[2] = 400; successor[2] = 399;
		current[3] = 399; successor[3] = 558;
		current[4] = 558; successor[4] = 557;
		current[5] = 557; successor[5] = 716;
		current[6] = 716; successor[6] = 715;
		current[7] = 715; successor[7] = 874;
		current[8] = 874; successor[8] = 873;
		current[9] = 873; successor[9] = 1032;
		current[10] = 1032; successor[10] = 1031;
		current[11] = 1031; successor[11] = 1190;
		current[12] = 1190; successor[12] = 1189;
		current[13] = 1189; successor[13] = 1348;
		current[14] = 1348; successor[14] = 1347;
		current[15] = 1347; successor[15] = 1506;
		current[16] = 1506; successor[16] = 1505;
		current[17] = 1505; successor[17] = 1664;
		current[18] = 1664; successor[18] = 1663;
		current[19] = 1663; successor[19] = 1822;
		current[20] = 1822; successor[20] = 1821;
		current[21] = 1821; successor[21] = 1981;
		current[22] = 1981; successor[22] = 2141;
		current[23] = 2141; successor[23] = 2301;
		current[24] = 2301; successor[24] = 2461;
		current[25] = 2461; successor[25] = 2621;
		current[26] = 2621; successor[26] = 2781;
		current[27] = 2781; successor[27] = 2941;
		current[28] = 2941; successor[28] = 3101;
		current[29] = 3101; successor[29] = 3261;
		current[30] = 3261; successor[30] = 3421;
		current[31] = 3421; successor[31] = 3581;
		current[32] = 3581; successor[32] = 3741;
		current[33] = 3741; successor[33] = 3901;
		current[34] = 3901; successor[34] = 4061;
		current[35] = 4061; successor[35] = 4221;
		current[36] = 4221; successor[36] = 4381;
		current[37] = 4381; successor[37] = 4541;
		current[38] = 4541; successor[38] = 4701;
		current[39] = 4701; successor[39] = 4861;
		current[40] = 4861; successor[40] = 5021;
		current[41] = 5021; successor[41] = 5181;
		current[42] = 5181; successor[42] = 5341;
		current[43] = 5341; successor[43] = 5501;
		current[44] = 5501; successor[44] = 5661;
		current[45] = 5661; successor[45] = 5821;
		current[46] = 5821; successor[46] = 5981;
		current[47] = 5981; successor[47] = 6141;
		current[48] = 6141; successor[48] = 6301;
		current[49] = 6301; successor[49] = 6461;
		current[50] = 6461; successor[50] = 6621;
		current[51] = 6621; successor[51] = 6781;
		current[52] = 6781; successor[52] = 6941;
		current[53] = 6941; successor[53] = 7101;
		current[54] = 7101; successor[54] = 7261;
		current[55] = 7261; successor[55] = 7421;
		current[56] = 7421; successor[56] = 7581;
		current[57] = 7581; successor[57] = 7741;
		current[58] = 7741; successor[58] = 7901;
		current[59] = 7901; successor[59] = 8061;
		current[60] = 8061; successor[60] = 8221;
		current[61] = 8221; successor[61] = 8381;
		current[62] = 8381; successor[62] = 8541;
		current[63] = 8541; successor[63] = 8701;
		current[64] = 8701; successor[64] = 8861;
		current[65] = 8861; successor[65] = 9021;
		current[66] = 9021; successor[66] = 9181;
		current[67] = 9181; successor[67] = 9341;
		current[68] = 9341; successor[68] = 9501;
		current[69] = 9501; successor[69] = 9661;
		current[70] = 9661; successor[70] = 9821;
		current[71] = 9821; successor[71] = 9981;
		current[72] = 9981; successor[72] = 10141;
		current[73] = 10141; successor[73] = 10301;
		current[74] = 10301; successor[74] = 10461;
		current[75] = 10461; successor[75] = 10621;
		current[76] = 10621; successor[76] = 10781;
		current[77] = 10781; successor[77] = 10941;
		current[78] = 10941; successor[78] = 11101;
		current[79] = 11101; successor[79] = 11261;
		current[80] = 11261; successor[80] = 11421;
		current[81] = 11421; successor[81] = 11581;
		current[82] = 11581; successor[82] = 11741;
		current[83] = 11741; successor[83] = 11901;
		current[84] = 11901; successor[84] = 12061;
		current[85] = 12061; successor[85] = 12221;
		current[86] = 12221; successor[86] = 12381;
		current[87] = 12381; successor[87] = 12541;
		current[88] = 12541; successor[88] = 12701;
		current[89] = 12701; successor[89] = 12861;
		current[90] = 12861; successor[90] = 13021;
		current[91] = 13021; successor[91] = 13181;
		current[92] = 13181; successor[92] = 13341;
		current[93] = 13341; successor[93] = 13501;
		current[94] = 13501; successor[94] = 13661;
		current[95] = 13661; successor[95] = 13821;
		current[96] = 13821; successor[96] = 13981;
		current[97] = 13981; successor[97] = 14141;
		current[98] = 14141; successor[98] = 14301;
		current[99] = 14301; successor[99] = 14461;
		current[100] = 14461; successor[100] = 14621;
		current[101] = 14621; successor[101] = 14781;
		current[102] = 14781; successor[102] = 14941;
		current[103] = 14941; successor[103] = 15101;
		current[104] = 15101; successor[104] = 15261;
		current[105] = 15261; successor[105] = 15421;
		current[106] = 15421; successor[106] = 15581;
		current[107] = 15581; successor[107] = 15741;
		current[108] = 15741; successor[108] = 15901;
		current[109] = 15901; successor[109] = 16061;
		current[110] = 16061; successor[110] = 16060;
		current[111] = 16060; successor[111] = 16059;
		current[112] = 16059; successor[112] = 16058;
		current[113] = 16058; successor[113] = 16057;
		current[114] = 16057; successor[114] = 16056;
		current[115] = 16056; successor[115] = 16055;
		current[116] = 16055; successor[116] = 16054;
		current[117] = 16054; successor[117] = 16053;
		current[118] = 16053; successor[118] = 16052;
		current[119] = 16052; successor[119] = 16051;
		current[120] = 16051; successor[120] = 16050;
		current[121] = 16050; successor[121] = 16049;
		current[122] = 16049; successor[122] = 16048;
		current[123] = 16048; successor[123] = 16047;
		current[124] = 16047; successor[124] = 16046;
		current[125] = 16046; successor[125] = 16045;
		current[126] = 16045; successor[126] = 16044;
		current[127] = 16044; successor[127] = 16043;
		current[128] = 16043; successor[128] = 16042;
		current[129] = 16042; successor[129] = 16041;
		current[130] = 16041; successor[130] = 16040;
		current[131] = 16040; successor[131] = 16039;
		current[132] = 16039; successor[132] = 16038;
		current[133] = 16038; successor[133] = 16037;
		current[134] = 16037; successor[134] = 16036;
		current[135] = 16036; successor[135] = 16035;
		current[136] = 16035; successor[136] = 16034;
		current[137] = 16034; successor[137] = 16194;
		current[138] = 16194; successor[138] = 16354;
		current[139] = 16354; successor[139] = 16514;
		current[140] = 16514; successor[140] = 16674;
		current[141] = 16674; successor[141] = 16834;
		current[142] = 16834; successor[142] = 16994;
		current[143] = 16994; successor[143] = 17154;
		current[144] = 17154; successor[144] = 17314;
		current[145] = 17314; successor[145] = 17474;
		current[146] = 17474; successor[146] = 17634;
		current[147] = 17634; successor[147] = 17794;
		current[148] = 17794; successor[148] = 17954;
		current[149] = 17954; successor[149] = 18114;
		current[150] = 18114; successor[150] = 18274;
		current[151] = 18274; successor[151] = 18434;
		current[152] = 18434; successor[152] = 18594;
		current[153] = 18594; successor[153] = 18754;
		current[154] = 18754; successor[154] = 18914;
		current[155] = 18914; successor[155] = 19074;
		current[156] = 19074; successor[156] = 19234;
		current[157] = 19234; successor[157] = 19394;
		current[158] = 19394; successor[158] = 19554;
		current[159] = 19554; successor[159] = 19714;
		current[160] = 19714; successor[160] = 19874;
		current[161] = 19874; successor[161] = 20034;
		current[162] = 20034; successor[162] = 20194;
		current[163] = 20194; successor[163] = 20354;
		current[164] = 20354; successor[164] = 20514;
		current[165] = 20514; successor[165] = 20674;
		current[166] = 20674; successor[166] = 20834;
		current[167] = 20834; successor[167] = 20994;
		current[168] = 20994; successor[168] = 21154;
		current[169] = 21154; successor[169] = 21314;
		current[170] = 21314; successor[170] = 21474;
		current[171] = 21474; successor[171] = 21634;
		current[172] = 21634; successor[172] = 21794;
		current[173] = 21794; successor[173] = 21954;
		current[174] = 21954; successor[174] = 22114;
		current[175] = 22114; successor[175] = 22274;
		current[176] = 22274; successor[176] = 22434;
		current[177] = 22434; successor[177] = 22594;
		current[178] = 22594; successor[178] = 22754;
		current[179] = 22754; successor[179] = 22914;
		current[180] = 22914; successor[180] = 23074;
		current[181] = 23074; successor[181] = 23234;
		current[182] = 23234; successor[182] = 23394;
		current[183] = 23394; successor[183] = 23554;
		current[184] = 23554; successor[184] = 23714;
		current[185] = 23714; successor[185] = 23874;
		current[186] = 23874; successor[186] = 24034;
		current[187] = 24034; successor[187] = 24194;
		current[188] = 24194; successor[188] = 24354;
		current[189] = 24354; successor[189] = 24514;
		current[190] = 24514; successor[190] = 24674;
		current[191] = 24674; successor[191] = 24834;
		current[192] = 24834; successor[192] = 24994;
		current[193] = 24994; successor[193] = 25154;
		current[194] = 25154; successor[194] = 25314;
		current[195] = 25314; successor[195] = 25474;
		current[196] = 25474; successor[196] = 25634;
		current[197] = 25634; successor[197] = 25794;
		current[198] = 25794; successor[198] = 25954;
		current[199] = 25954; successor[199] = 26114;
		current[200] = 26114; successor[200] = 26274;
		current[201] = 26274; successor[201] = 26434;
		current[202] = 26434; successor[202] = 26594;
		current[203] = 26594; successor[203] = 26754;
		current[204] = 26754; successor[204] = 26914;
		current[205] = 26914; successor[205] = 27074;
		current[206] = 27074; successor[206] = 27234;
		current[207] = 27234; successor[207] = 27394;
		current[208] = 27394; successor[208] = 27554;
		current[209] = 27554; successor[209] = 27714;
		current[210] = 27714; successor[210] = 27874;
		current[211] = 27874; successor[211] = 28034;
		current[212] = 28034; successor[212] = 28194;
		current[213] = 28194; successor[213] = 28354;
		current[214] = 28354; successor[214] = 28514;
		current[215] = 28514; successor[215] = 28674;
		current[216] = 28674; successor[216] = 28834;
		current[217] = 28834; successor[217] = 28994;
		current[218] = 28994; successor[218] = 29154;
		current[219] = 29154; successor[219] = 29314;
		current[220] = 29314; successor[220] = 29474;
		current[221] = 29474; successor[221] = 29634;
		current[222] = 29634; successor[222] = 29794;
		current[223] = 29794; successor[223] = 29954;
		current[224] = 29954; successor[224] = 30114;
		current[225] = 30114; successor[225] = 30274;
		current[226] = 30274; successor[226] = 30434;
		current[227] = 30434; successor[227] = 30594;
		current[228] = 30594; successor[228] = 30754;
		current[229] = 30754; successor[229] = 30914;
		current[230] = 30914; successor[230] = 31074;
		current[231] = 31074; successor[231] = 31234;
		current[232] = 31234; successor[232] = 31394;
		current[233] = 31394; successor[233] = 31554;
		current[234] = 31554; successor[234] = 31714;
		current[235] = 31714; successor[235] = 31874;
		current[236] = 31874; successor[236] = 32034;
		current[237] = 32034; successor[237] = 32194;
		current[238] = 32194; successor[238] = 32354;
		current[239] = 32354; successor[239] = 32514;
		current[240] = 32514; successor[240] = 32674;
		break;
	case 5:
		sol.road.source = 27;
		sol.road.end = 4547;

		current[0] = 27; successor[0] = 80;
		current[1] = 80; successor[1] = 79;
		current[2] = 79; successor[2] = 132;
		current[3] = 132; successor[3] = 131;
		current[4] = 131; successor[4] = 184;
		current[5] = 184; successor[5] = 183;
		current[6] = 183; successor[6] = 236;
		current[7] = 236; successor[7] = 290;
		current[8] = 290; successor[8] = 344;
		current[9] = 344; successor[9] = 398;
		current[10] = 398; successor[10] = 452;
		current[11] = 452; successor[11] = 506;
		current[12] = 506; successor[12] = 560;
		current[13] = 560; successor[13] = 614;
		current[14] = 614; successor[14] = 668;
		current[15] = 668; successor[15] = 722;
		current[16] = 722; successor[16] = 776;
		current[17] = 776; successor[17] = 830;
		current[18] = 830; successor[18] = 884;
		current[19] = 884; successor[19] = 938;
		current[20] = 938; successor[20] = 992;
		current[21] = 992; successor[21] = 1046;
		current[22] = 1046; successor[22] = 1100;
		current[23] = 1100; successor[23] = 1154;
		current[24] = 1154; successor[24] = 1208;
		current[25] = 1208; successor[25] = 1262;
		current[26] = 1262; successor[26] = 1316;
		current[27] = 1316; successor[27] = 1370;
		current[28] = 1370; successor[28] = 1424;
		current[29] = 1424; successor[29] = 1478;
		current[30] = 1478; successor[30] = 1532;
		current[31] = 1532; successor[31] = 1586;
		current[32] = 1586; successor[32] = 1640;
		current[33] = 1640; successor[33] = 1694;
		current[34] = 1694; successor[34] = 1748;
		current[35] = 1748; successor[35] = 1802;
		current[36] = 1802; successor[36] = 1801;
		current[37] = 1801; successor[37] = 1800;
		current[38] = 1800; successor[38] = 1799;
		current[39] = 1799; successor[39] = 1798;
		current[40] = 1798; successor[40] = 1797;
		current[41] = 1797; successor[41] = 1796;
		current[42] = 1796; successor[42] = 1795;
		current[43] = 1795; successor[43] = 1794;
		current[44] = 1794; successor[44] = 1793;
		current[45] = 1793; successor[45] = 1847;
		current[46] = 1847; successor[46] = 1901;
		current[47] = 1901; successor[47] = 1955;
		current[48] = 1955; successor[48] = 2009;
		current[49] = 2009; successor[49] = 2063;
		current[50] = 2063; successor[50] = 2117;
		current[51] = 2117; successor[51] = 2171;
		current[52] = 2171; successor[52] = 2225;
		current[53] = 2225; successor[53] = 2279;
		current[54] = 2279; successor[54] = 2333;
		current[55] = 2333; successor[55] = 2387;
		current[56] = 2387; successor[56] = 2441;
		current[57] = 2441; successor[57] = 2495;
		current[58] = 2495; successor[58] = 2549;
		current[59] = 2549; successor[59] = 2603;
		current[60] = 2603; successor[60] = 2657;
		current[61] = 2657; successor[61] = 2711;
		current[62] = 2711; successor[62] = 2765;
		current[63] = 2765; successor[63] = 2819;
		current[64] = 2819; successor[64] = 2873;
		current[65] = 2873; successor[65] = 2927;
		current[66] = 2927; successor[66] = 2981;
		current[67] = 2981; successor[67] = 3035;
		current[68] = 3035; successor[68] = 3089;
		current[69] = 3089; successor[69] = 3143;
		current[70] = 3143; successor[70] = 3197;
		current[71] = 3197; successor[71] = 3251;
		current[72] = 3251; successor[72] = 3305;
		current[73] = 3305; successor[73] = 3359;
		current[74] = 3359; successor[74] = 3413;
		current[75] = 3413; successor[75] = 3467;
		current[76] = 3467; successor[76] = 3521;
		current[77] = 3521; successor[77] = 3575;
		current[78] = 3575; successor[78] = 3629;
		current[79] = 3629; successor[79] = 3713;
		current[80] = 3713; successor[80] = 3797;
		current[81] = 3797; successor[81] = 3881;
		current[82] = 3881; successor[82] = 3966;
		current[83] = 3966; successor[83] = 4051;
		current[84] = 4051; successor[84] = 4136;
		current[85] = 4136; successor[85] = 4221;
		current[86] = 4221; successor[86] = 4306;
		current[87] = 4306; successor[87] = 4391;
		current[88] = 4391; successor[88] = 4477;
		current[89] = 4477; successor[89] = 4563;
		current[90] = 4563; successor[90] = 4562;
		current[91] = 4562; successor[91] = 4561;
		current[92] = 4561; successor[92] = 4560;
		current[93] = 4560; successor[93] = 4559;
		current[94] = 4559; successor[94] = 4558;
		current[95] = 4558; successor[95] = 4557;
		current[96] = 4557; successor[96] = 4556;
		current[97] = 4556; successor[97] = 4555;
		current[98] = 4555; successor[98] = 4554;
		current[99] = 4554; successor[99] = 4553;
		current[100] = 4553; successor[100] = 4552;
		current[101] = 4552; successor[101] = 4551;
		current[102] = 4551; successor[102] = 4550;
		current[103] = 4550; successor[103] = 4549;
		current[104] = 4549; successor[104] = 4548;
		current[105] = 4548; successor[105] = 4547;
		break;
	case 6:
		sol.road.source = 82;
		sol.road.end = 40460;

		current[0] = 82; successor[0] = 241;
		current[1] = 241; successor[1] = 400;
		current[2] = 400; successor[2] = 399;
		current[3] = 399; successor[3] = 558;
		current[4] = 558; successor[4] = 557;
		current[5] = 557; successor[5] = 716;
		current[6] = 716; successor[6] = 715;
		current[7] = 715; successor[7] = 874;
		current[8] = 874; successor[8] = 873;
		current[9] = 873; successor[9] = 1032;
		current[10] = 1032; successor[10] = 1031;
		current[11] = 1031; successor[11] = 1190;
		current[12] = 1190; successor[12] = 1189;
		current[13] = 1189; successor[13] = 1348;
		current[14] = 1348; successor[14] = 1347;
		current[15] = 1347; successor[15] = 1506;
		current[16] = 1506; successor[16] = 1505;
		current[17] = 1505; successor[17] = 1664;
		current[18] = 1664; successor[18] = 1663;
		current[19] = 1663; successor[19] = 1822;
		current[20] = 1822; successor[20] = 1821;
		current[21] = 1821; successor[21] = 1981;
		current[22] = 1981; successor[22] = 2141;
		current[23] = 2141; successor[23] = 2301;
		current[24] = 2301; successor[24] = 2461;
		current[25] = 2461; successor[25] = 2621;
		current[26] = 2621; successor[26] = 2781;
		current[27] = 2781; successor[27] = 2941;
		current[28] = 2941; successor[28] = 3101;
		current[29] = 3101; successor[29] = 3261;
		current[30] = 3261; successor[30] = 3421;
		current[31] = 3421; successor[31] = 3581;
		current[32] = 3581; successor[32] = 3741;
		current[33] = 3741; successor[33] = 3901;
		current[34] = 3901; successor[34] = 4061;
		current[35] = 4061; successor[35] = 4221;
		current[36] = 4221; successor[36] = 4381;
		current[37] = 4381; successor[37] = 4541;
		current[38] = 4541; successor[38] = 4701;
		current[39] = 4701; successor[39] = 4861;
		current[40] = 4861; successor[40] = 5021;
		current[41] = 5021; successor[41] = 5181;
		current[42] = 5181; successor[42] = 5341;
		current[43] = 5341; successor[43] = 5501;
		current[44] = 5501; successor[44] = 5661;
		current[45] = 5661; successor[45] = 5821;
		current[46] = 5821; successor[46] = 5981;
		current[47] = 5981; successor[47] = 6141;
		current[48] = 6141; successor[48] = 6301;
		current[49] = 6301; successor[49] = 6461;
		current[50] = 6461; successor[50] = 6621;
		current[51] = 6621; successor[51] = 6781;
		current[52] = 6781; successor[52] = 6941;
		current[53] = 6941; successor[53] = 7101;
		current[54] = 7101; successor[54] = 7261;
		current[55] = 7261; successor[55] = 7421;
		current[56] = 7421; successor[56] = 7581;
		current[57] = 7581; successor[57] = 7741;
		current[58] = 7741; successor[58] = 7901;
		current[59] = 7901; successor[59] = 8061;
		current[60] = 8061; successor[60] = 8221;
		current[61] = 8221; successor[61] = 8381;
		current[62] = 8381; successor[62] = 8541;
		current[63] = 8541; successor[63] = 8701;
		current[64] = 8701; successor[64] = 8861;
		current[65] = 8861; successor[65] = 9021;
		current[66] = 9021; successor[66] = 9181;
		current[67] = 9181; successor[67] = 9341;
		current[68] = 9341; successor[68] = 9501;
		current[69] = 9501; successor[69] = 9661;
		current[70] = 9661; successor[70] = 9821;
		current[71] = 9821; successor[71] = 9981;
		current[72] = 9981; successor[72] = 10141;
		current[73] = 10141; successor[73] = 10301;
		current[74] = 10301; successor[74] = 10461;
		current[75] = 10461; successor[75] = 10621;
		current[76] = 10621; successor[76] = 10781;
		current[77] = 10781; successor[77] = 10941;
		current[78] = 10941; successor[78] = 11101;
		current[79] = 11101; successor[79] = 11261;
		current[80] = 11261; successor[80] = 11421;
		current[81] = 11421; successor[81] = 11581;
		current[82] = 11581; successor[82] = 11741;
		current[83] = 11741; successor[83] = 11901;
		current[84] = 11901; successor[84] = 12061;
		current[85] = 12061; successor[85] = 12221;
		current[86] = 12221; successor[86] = 12381;
		current[87] = 12381; successor[87] = 12541;
		current[88] = 12541; successor[88] = 12701;
		current[89] = 12701; successor[89] = 12861;
		current[90] = 12861; successor[90] = 13021;
		current[91] = 13021; successor[91] = 13181;
		current[92] = 13181; successor[92] = 13341;
		current[93] = 13341; successor[93] = 13501;
		current[94] = 13501; successor[94] = 13661;
		current[95] = 13661; successor[95] = 13821;
		current[96] = 13821; successor[96] = 13981;
		current[97] = 13981; successor[97] = 14141;
		current[98] = 14141; successor[98] = 14301;
		current[99] = 14301; successor[99] = 14461;
		current[100] = 14461; successor[100] = 14621;
		current[101] = 14621; successor[101] = 14781;
		current[102] = 14781; successor[102] = 14941;
		current[103] = 14941; successor[103] = 15101;
		current[104] = 15101; successor[104] = 15261;
		current[105] = 15261; successor[105] = 15421;
		current[106] = 15421; successor[106] = 15581;
		current[107] = 15581; successor[107] = 15741;
		current[108] = 15741; successor[108] = 15901;
		current[109] = 15901; successor[109] = 16061;
		current[110] = 16061; successor[110] = 16060;
		current[111] = 16060; successor[111] = 16059;
		current[112] = 16059; successor[112] = 16058;
		current[113] = 16058; successor[113] = 16057;
		current[114] = 16057; successor[114] = 16056;
		current[115] = 16056; successor[115] = 16055;
		current[116] = 16055; successor[116] = 16054;
		current[117] = 16054; successor[117] = 16053;
		current[118] = 16053; successor[118] = 16052;
		current[119] = 16052; successor[119] = 16051;
		current[120] = 16051; successor[120] = 16050;
		current[121] = 16050; successor[121] = 16049;
		current[122] = 16049; successor[122] = 16048;
		current[123] = 16048; successor[123] = 16047;
		current[124] = 16047; successor[124] = 16046;
		current[125] = 16046; successor[125] = 16045;
		current[126] = 16045; successor[126] = 16044;
		current[127] = 16044; successor[127] = 16043;
		current[128] = 16043; successor[128] = 16042;
		current[129] = 16042; successor[129] = 16041;
		current[130] = 16041; successor[130] = 16040;
		current[131] = 16040; successor[131] = 16039;
		current[132] = 16039; successor[132] = 16038;
		current[133] = 16038; successor[133] = 16037;
		current[134] = 16037; successor[134] = 16036;
		current[135] = 16036; successor[135] = 16035;
		current[136] = 16035; successor[136] = 16034;
		current[137] = 16034; successor[137] = 16194;
		current[138] = 16194; successor[138] = 16354;
		current[139] = 16354; successor[139] = 16514;
		current[140] = 16514; successor[140] = 16674;
		current[141] = 16674; successor[141] = 16834;
		current[142] = 16834; successor[142] = 16994;
		current[143] = 16994; successor[143] = 17154;
		current[144] = 17154; successor[144] = 17314;
		current[145] = 17314; successor[145] = 17474;
		current[146] = 17474; successor[146] = 17634;
		current[147] = 17634; successor[147] = 17794;
		current[148] = 17794; successor[148] = 17954;
		current[149] = 17954; successor[149] = 18114;
		current[150] = 18114; successor[150] = 18274;
		current[151] = 18274; successor[151] = 18434;
		current[152] = 18434; successor[152] = 18594;
		current[153] = 18594; successor[153] = 18754;
		current[154] = 18754; successor[154] = 18914;
		current[155] = 18914; successor[155] = 19074;
		current[156] = 19074; successor[156] = 19234;
		current[157] = 19234; successor[157] = 19394;
		current[158] = 19394; successor[158] = 19554;
		current[159] = 19554; successor[159] = 19714;
		current[160] = 19714; successor[160] = 19874;
		current[161] = 19874; successor[161] = 20034;
		current[162] = 20034; successor[162] = 20194;
		current[163] = 20194; successor[163] = 20354;
		current[164] = 20354; successor[164] = 20514;
		current[165] = 20514; successor[165] = 20674;
		current[166] = 20674; successor[166] = 20834;
		current[167] = 20834; successor[167] = 20994;
		current[168] = 20994; successor[168] = 21154;
		current[169] = 21154; successor[169] = 21314;
		current[170] = 21314; successor[170] = 21474;
		current[171] = 21474; successor[171] = 21634;
		current[172] = 21634; successor[172] = 21794;
		current[173] = 21794; successor[173] = 21954;
		current[174] = 21954; successor[174] = 22114;
		current[175] = 22114; successor[175] = 22274;
		current[176] = 22274; successor[176] = 22434;
		current[177] = 22434; successor[177] = 22594;
		current[178] = 22594; successor[178] = 22754;
		current[179] = 22754; successor[179] = 22914;
		current[180] = 22914; successor[180] = 23074;
		current[181] = 23074; successor[181] = 23234;
		current[182] = 23234; successor[182] = 23394;
		current[183] = 23394; successor[183] = 23554;
		current[184] = 23554; successor[184] = 23714;
		current[185] = 23714; successor[185] = 23874;
		current[186] = 23874; successor[186] = 24034;
		current[187] = 24034; successor[187] = 24194;
		current[188] = 24194; successor[188] = 24354;
		current[189] = 24354; successor[189] = 24514;
		current[190] = 24514; successor[190] = 24674;
		current[191] = 24674; successor[191] = 24834;
		current[192] = 24834; successor[192] = 24994;
		current[193] = 24994; successor[193] = 25154;
		current[194] = 25154; successor[194] = 25314;
		current[195] = 25314; successor[195] = 25474;
		current[196] = 25474; successor[196] = 25634;
		current[197] = 25634; successor[197] = 25794;
		current[198] = 25794; successor[198] = 25954;
		current[199] = 25954; successor[199] = 26114;
		current[200] = 26114; successor[200] = 26274;
		current[201] = 26274; successor[201] = 26434;
		current[202] = 26434; successor[202] = 26594;
		current[203] = 26594; successor[203] = 26754;
		current[204] = 26754; successor[204] = 26914;
		current[205] = 26914; successor[205] = 27074;
		current[206] = 27074; successor[206] = 27234;
		current[207] = 27234; successor[207] = 27394;
		current[208] = 27394; successor[208] = 27554;
		current[209] = 27554; successor[209] = 27714;
		current[210] = 27714; successor[210] = 27874;
		current[211] = 27874; successor[211] = 28034;
		current[212] = 28034; successor[212] = 28194;
		current[213] = 28194; successor[213] = 28354;
		current[214] = 28354; successor[214] = 28514;
		current[215] = 28514; successor[215] = 28674;
		current[216] = 28674; successor[216] = 28834;
		current[217] = 28834; successor[217] = 28994;
		current[218] = 28994; successor[218] = 29154;
		current[219] = 29154; successor[219] = 29314;
		current[220] = 29314; successor[220] = 29474;
		current[221] = 29474; successor[221] = 29634;
		current[222] = 29634; successor[222] = 29794;
		current[223] = 29794; successor[223] = 29954;
		current[224] = 29954; successor[224] = 30114;
		current[225] = 30114; successor[225] = 30274;
		current[226] = 30274; successor[226] = 30434;
		current[227] = 30434; successor[227] = 30594;
		current[228] = 30594; successor[228] = 30754;
		current[229] = 30754; successor[229] = 30914;
		current[230] = 30914; successor[230] = 31074;
		current[231] = 31074; successor[231] = 31234;
		current[232] = 31234; successor[232] = 31394;
		current[233] = 31394; successor[233] = 31554;
		current[234] = 31554; successor[234] = 31714;
		current[235] = 31714; successor[235] = 31874;
		current[236] = 31874; successor[236] = 32034;
		current[237] = 32034; successor[237] = 32194;
		current[238] = 32194; successor[238] = 32354;
		current[239] = 32354; successor[239] = 32514;
		current[240] = 32514; successor[240] = 32674;
		current[241] = 32674; successor[241] = 32924;
		current[242] = 32924; successor[242] = 33174;
		current[243] = 33174; successor[243] = 33424;
		current[244] = 33424; successor[244] = 33675;
		current[245] = 33675; successor[245] = 33926;
		current[246] = 33926; successor[246] = 34177;
		current[247] = 34177; successor[247] = 34428;
		current[248] = 34428; successor[248] = 34679;
		current[249] = 34679; successor[249] = 34930;
		current[250] = 34930; successor[250] = 35182;
		current[251] = 35182; successor[251] = 35434;
		current[252] = 35434; successor[252] = 35686;
		current[253] = 35686; successor[253] = 35938;
		current[254] = 35938; successor[254] = 36190;
		current[255] = 36190; successor[255] = 36443;
		current[256] = 36443; successor[256] = 36696;
		current[257] = 36696; successor[257] = 36949;
		current[258] = 36949; successor[258] = 37202;
		current[259] = 37202; successor[259] = 37455;
		current[260] = 37455; successor[260] = 37708;
		current[261] = 37708; successor[261] = 37962;
		current[262] = 37962; successor[262] = 38216;
		current[263] = 38216; successor[263] = 38470;
		current[264] = 38470; successor[264] = 38724;
		current[265] = 38724; successor[265] = 38978;
		current[266] = 38978; successor[266] = 39233;
		current[267] = 39233; successor[267] = 39488;
		current[268] = 39488; successor[268] = 39743;
		current[269] = 39743; successor[269] = 39998;
		current[270] = 39998; successor[270] = 40253;
		current[271] = 40253; successor[271] = 40508;
		current[272] = 40508; successor[272] = 40507;
		current[273] = 40507; successor[273] = 40506;
		current[274] = 40506; successor[274] = 40505;
		current[275] = 40505; successor[275] = 40504;
		current[276] = 40504; successor[276] = 40503;
		current[277] = 40503; successor[277] = 40502;
		current[278] = 40502; successor[278] = 40501;
		current[279] = 40501; successor[279] = 40500;
		current[280] = 40500; successor[280] = 40499;
		current[281] = 40499; successor[281] = 40498;
		current[282] = 40498; successor[282] = 40497;
		current[283] = 40497; successor[283] = 40496;
		current[284] = 40496; successor[284] = 40495;
		current[285] = 40495; successor[285] = 40494;
		current[286] = 40494; successor[286] = 40493;
		current[287] = 40493; successor[287] = 40492;
		current[288] = 40492; successor[288] = 40491;
		current[289] = 40491; successor[289] = 40490;
		current[290] = 40490; successor[290] = 40489;
		current[291] = 40489; successor[291] = 40488;
		current[292] = 40488; successor[292] = 40487;
		current[293] = 40487; successor[293] = 40486;
		current[294] = 40486; successor[294] = 40485;
		current[295] = 40485; successor[295] = 40484;
		current[296] = 40484; successor[296] = 40483;
		current[297] = 40483; successor[297] = 40482;
		current[298] = 40482; successor[298] = 40481;
		current[299] = 40481; successor[299] = 40480;
		current[300] = 40480; successor[300] = 40479;
		current[301] = 40479; successor[301] = 40478;
		current[302] = 40478; successor[302] = 40477;
		current[303] = 40477; successor[303] = 40476;
		current[304] = 40476; successor[304] = 40475;
		current[305] = 40475; successor[305] = 40474;
		current[306] = 40474; successor[306] = 40473;
		current[307] = 40473; successor[307] = 40472;
		current[308] = 40472; successor[308] = 40471;
		current[309] = 40471; successor[309] = 40470;
		current[310] = 40470; successor[310] = 40469;
		current[311] = 40469; successor[311] = 40468;
		current[312] = 40468; successor[312] = 40467;
		current[313] = 40467; successor[313] = 40466;
		current[314] = 40466; successor[314] = 40465;
		current[315] = 40465; successor[315] = 40464;
		current[316] = 40464; successor[316] = 40463;
		current[317] = 40463; successor[317] = 40462;
		current[318] = 40462; successor[318] = 40461;
		current[319] = 40461; successor[319] = 40460;
		break;
	default:
		break;
	}

	sol.distance[sol.road.source] = 0;
	sol.weight[sol.road.end] = 0;

	for (i = 0; i < size; i++) {
		sol.distance[successor[i]] = sol.distance[current[i]] + CalculatesDistance(ar, current[i], successor[i], 1);
		sol.weight[successor[i]] = sol.weight[current[i]] + CalculatesPenaltyDistance(ar, current[i], successor[i], turn, PPA, inund, slope);
		sol.ancestor[successor[i]] = current[i];
		sol.successor[current[i]] = successor[i];
	}

	sol.FO = sol.weight[sol.road.end];
	sol.totalDistance = sol.distance[sol.road.end];
	sol.time = 0;
	sol.routeNumVertex = size;
	sol.timeSol = sol.time;

	return sol;
}