#include "stdafx.h"
#include <iostream>
#include <string.h>
#include <time.h>
#include "Graph.h"
#include "Commons.h"
#include "ExecMHRoad.h"
#include "TimeCounter.h"

using namespace std;

//definition constants of which algorithms will be executed
#define execDijkstra 1
//#undef execDijkstra
#define execDijkstraOD 1
//#undef execDijkstraOD
#define execBellmanFord 1
//#undef execBellmanFord
#define execDial 1
//#undef execDial
#define execDEsopoPape 1
//#undef execDEsopoPape
#define execForestCompany 1
//#undef execForestCompany

//constant to define the calibration execution
#define tunning 1
//#undef tunning

//definition constants of which instance will be calibrated
#define execCalInst5 1
#undef execCalInst5
#define execCalInst6 1
#undef execCalInst6

int _tmain(int argc, _TCHAR* argv[])
{
	int i;
	char file[50], resul_file[50], sol_file[50], summary_file[50], constraint_file[50], numArq[8];
	printTime();
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	//--------------------------PRE-PROCESSING ROADS -------------------------------//
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//

	//--------------------- importing the Graph instance ------------------
	Area *ar;
	cout << "Definindo o nome do arquivo da instancia do Graph para leitura..." << endl;
	strcpy(file, "D:\\");
	strcat(file, INST_G);
	strcat(file, ".txt");
	cout << "Lendo os dados da instancia do Graph..." << endl;
	ar = importGraphAreaInstance(file);

	//--------------------- importing the PPA instance ------------------
	int *PPA;
	cout << "Definindo o nome do arquivo da instancia da PPA para leitura..." << endl;
	strcpy(file, "D:\\");
	strcat(file, INST_PPA);
	strcat(file, ".txt");
	cout << "Lendo os dados da instancia da PPA..." << endl;
	PPA = importPPAInstance(file);

	//--------------------- IMPORTING THE INUND INSTANCE ------------------
	int *inund;
	cout << "Definindo o nome do arquivo da instancia de inundacao para leitura..." << endl;
	strcpy(file, "D:\\");
	strcat(file, INST_INUND);
	strcat(file, ".txt");
	cout << "Lendo os dados da instancia da inundacao..." << endl;
	inund = importInundInstance(file);

	//--------------------- IMPORTING THE SLOPE INSTANCE ------------------
	Slope *slope;
	cout << "Definindo o nome do arquivo da instancia de slope para leitura..." << endl;
	strcpy(file, "D:\\");
	strcat(file, INST_SLOPE);
	strcat(file, ".txt");
	cout << "Lendo os dados da instancia da slope..." << endl;
	slope = importSlopeInstance(file);

	//--------------------- IMPORTING THE SOURCERIVER INSTANCE ------------------
	Turn turn[NUM_TURN];
#ifdef NUM_SOURCERIVER
	SourceRiver sourceRiver[NUM_SOURCERIVER];
	if (NUM_SOURCERIVER > 0) {
		cout << "Definindo o nome do arquivo da instancia das nascentes para leitura..." << endl;
		strcpy(file, "D:\\");
		strcat(file, INST_SOURCERIVER);
		strcat(file, ".txt");
		cout << "Lendo os dados da instancia das nascentes..." << endl;
		importSourceRiverInstance(file, sourceRiver, turn);
	}
#endif

//--------------------- IMPORTING THE TREE INSTANCE ------------------
cout << "Definindo o nome do arquivo da instancia das arvores remanescentes para leitura..." << endl;
Tree tree[NUM_TREE];
strcpy(file, "D:\\");
strcat(file, INST_TREE);
strcat(file, ".txt");
cout << "Lendo os dados da instancia das arvores remanescentes..." << endl;
importTreesInstance(file, tree, turn);


//---------------------------- defining Graph ---------------------------------
cout << "Construindo o grafo..." << endl;
Graph *gr;
gr = buildGraph(NUM_VERTEX, MAX_NEIGHBOR, WEIGHTING);

cout << "Inserindo arestas no Graph." << endl;
insertAreaEdge(gr, ar, turn, PPA, inund, slope);
printTime();
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//--------------------------------------RODAS-----------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//

//defining the start and end points of the road
Road roads[NUM_ROAD];

#ifdef execInst1
	//Instance 1 tests - UT3 - pixel 30m
	roads[0].source = 27;
	roads[0].end = 1802;
#endif

#ifdef execInst2
	//Instance 2 tests - UT3 - pixel 10m
	roads[0].inicio = 82;
	roads[0].termino = 15901;
#endif

#ifdef execInst3
	//Instance 3 tests (UT3 e UT4) - pixel 30m
	roads[0].inicio = 27;
	roads[0].termino = 3683;
#endif

#ifdef execInst4
	//Instance 4 tests (UT3 e UT4) - pixel 10m
	roads[0].inicio = 82;
	roads[0].termino = 32674;
#endif

#ifdef execInst5
	//Instance 5 tests (UT3, UT4 e UT5) - pixel 30m
	roads[0].inicio = 27;
	roads[0].termino = 4547;
#endif

#ifdef execInst6
	//Instance 6 tests (UT3, UT4 e UT5) - pixel 10m
	roads[0].inicio = 82;
	roads[0].termino = 40460;
#endif

//calibration vertices
#ifdef execCalInst5
	//vertices of instance 5 calibration
	roads[0].source = 3716;
	roads[0].end = 6543;
#endif

#ifdef execCalInst6
	//vertices of instance 6 calibration
	roads[0].inicio = 32933;
	roads[0].termino = 58036;
#endif

#ifdef execForestCompany
	GroupRoadSolution solCompany;

	solCompany.roads[0] = FillsRoadCompany(INST_COMPANY, NUM_VERTEX_COMPANY, gr, ar, turn, PPA, inund, slope);
	solCompany.totalDistance = solCompany.roads[0].totalDistance;
	solCompany.TotalFO = solCompany.roads[0].FO;
	solCompany.totalRouteNumVertex = NUM_VERTEX_COMPANY;
	solCompany.totalTime = 0;

	resul_file[0] = '\0';
	strcpy(resul_file, "D:\\");
	strcat(resul_file, RESUL_ROAD);
	strcat(resul_file, "_Emp");
	strcat(resul_file, ".txt");

	sol_file[0] = '\0';
	strcpy(sol_file, "D:\\");
	strcat(sol_file, SOL_ROAD);
	strcat(sol_file, "_Emp");
	strcat(sol_file, ".txt");

	summary_file[0] = '\0';
	strcpy(summary_file, "D:\\");
	strcat(summary_file, SUMMARY_ROAD);
	strcat(summary_file, "_Emp");
	strcat(summary_file, ".txt");

	SaveResultsRoads(resul_file, sol_file, summary_file, &solCompany, 1);

	constraint_file[0] = '\0';
	strcpy(constraint_file, "D:\\");
	strcat(constraint_file, CONSTRAINT_ROAD);
	strcat(constraint_file, "_Emp");
	strcat(constraint_file, ".txt");

	SaveResultsConstraint(&solCompany, constraint_file, ar, turn, PPA, inund, slope);
#endif

#ifdef tunning
	cout << "Tunning " << endl;
	GroupRoadSolution solRoadTunning;
	//defining the name of the results file
	strcpy(resul_file, "D:\\");
	strcat(resul_file, RESUL_ROAD);
	strcat(resul_file, "_Dijkstra");
	strcat(resul_file, ".txt");
	//deleting the file if it exists
	remove(resul_file);

	//defining the name of the solution file
	strcpy(sol_file, "D:\\");
	strcat(sol_file, SOL_ROAD);
	strcat(sol_file, "_Dijkstra");
	strcat(sol_file, ".txt");
	//apagando o arquivo caso exista
	remove(sol_file);

	//defining the name of the consolidated results file
	strcpy(summary_file, "D:\\");
	strcat(summary_file, SUMMARY_ROAD);
	strcat(summary_file, "_Dijkstra");
	strcat(summary_file, ".txt");

	cout << "Executando o dijkstra." << endl;
	solRoadTunning = designRoad(cDijkstra, gr, roads, resul_file, sol_file, summary_file, 1);

	//building the name of the constraint file
	constraint_file[0] = '\0';
	strcpy(constraint_file, "D:\\");
	strcat(constraint_file, CONSTRAINT_ROAD);
	strcat(constraint_file, "_MC");
	strcat(constraint_file, ".txt");

	SaveResultsConstraint(&solRoadTunning, constraint_file, ar, turn, PPA, inund, slope);
#endif

//------------------------------------------------------------
	
for (i = 1; i <= NUM_TESTS; i++) {
	cout << "Calculando menor caminho das estradas n.: " << i << endl;

	GroupRoadSolution solRoad;

	#ifdef execDijkstra
		resul_file[0] = '\0';
		sol_file[0] = '\0';

		strcpy(resul_file, "D:\\");
		strcat(resul_file, RESUL_ROAD);
		strcat(resul_file, "_Dijkstra");
		sprintf(numArq, "_%d", i);
		strcat(resul_file, numArq);
		strcat(resul_file, ".txt");
		remove(resul_file);

		strcpy(sol_file, "D:\\");
		strcat(sol_file, SOL_ROAD);
		strcat(sol_file, "_Dijkstra");
		strcat(sol_file, ".txt");
		remove(sol_file);

		strcpy(summary_file, "D:\\");
		strcat(summary_file, SUMMARY_ROAD);
		strcat(summary_file, "_Dijkstra");
		strcat(summary_file, ".txt");

		cout << "Executando o dijkstra." << endl;
		solRoad = designRoad(cDijkstra, gr, roads, resul_file, sol_file, summary_file, i);
	#endif

	#ifdef execDijkstraOD
		resul_file[0] = '\0';
		sol_file[0] = '\0';

		strcpy(resul_file, "D:\\");
		strcat(resul_file, RESUL_ROAD);
		strcat(resul_file, "_DijkstraOD");
		sprintf(numArq, "_%d", i);
		strcat(resul_file, numArq);
		strcat(resul_file, ".txt");
		remove(resul_file);

		strcpy(sol_file, "D:\\");
		strcat(sol_file, SOL_ROAD);
		strcat(sol_file, "_DijkstraOD");
		strcat(sol_file, ".txt");
		remove(sol_file);

		strcpy(summary_file, "D:\\");
		strcat(summary_file, SUMMARY_ROAD);
		strcat(summary_file, "_DijkstraOD");
		strcat(summary_file, ".txt");

		cout << "Executando o Dijkstra OD." << endl;
		solRoad = designRoad(cDijkstraOD, gr, roads, resul_file, sol_file, summary_file, i);
	#endif

	#ifdef execBellmanFord
		resul_file[0] = '\0';
		sol_file[0] = '\0';

		strcpy(resul_file, "D:\\");
		strcat(resul_file, RESUL_ROAD);
		strcat(resul_file, "_bellmanFord");
		sprintf(numArq, "_%d", i);
		strcat(resul_file, numArq);
		strcat(resul_file, ".txt");
		remove(resul_file);

		strcpy(sol_file, "D:\\");
		strcat(sol_file, SOL_ROAD);
		strcat(sol_file, "_bellmanFord");
		strcat(sol_file, ".txt");
		remove(sol_file);

		strcpy(summary_file, "D:\\");
		strcat(summary_file, SUMMARY_ROAD);
		strcat(summary_file, "_bellmanFord");
		strcat(summary_file, ".txt");

		cout << "Executando o Bellman Ford." << endl;
		cout << "Calculando menor caminho das estradas n.: " << i << endl;
		solRoad = designRoad(cbellmanFord, gr, roads, resul_file, sol_file, summary_file, i);
	#endif

	#ifdef execDial
		resul_file[0] = '\0';
		sol_file[0] = '\0';

		strcpy(resul_file, "D:\\");
		strcat(resul_file, RESUL_ROAD);
		strcat(resul_file, "_dial");
		sprintf(numArq, "_%d", i);
		strcat(resul_file, numArq);
		strcat(resul_file, ".txt");
		remove(resul_file);

		strcpy(sol_file, "D:\\");
		strcat(sol_file, SOL_ROAD);
		strcat(sol_file, "_dial");
		strcat(sol_file, ".txt");
		remove(sol_file);

		strcpy(summary_file, "D:\\");
		strcat(summary_file, SUMMARY_ROAD);
		strcat(summary_file, "_dial");
		strcat(summary_file, ".txt");

		cout << "Executando o Dial.\n" << endl;
		solRoad = designRoad(cDijkstraDial, gr, roads, resul_file, sol_file, summary_file, i);
	#endif

	#ifdef execDEsopoPape
		resul_file[0] = '\0';
		sol_file[0] = '\0';

		strcpy(resul_file, "D:\\");
		strcat(resul_file, RESUL_ROAD);
		strcat(resul_file, "_pape");
		sprintf(numArq, "_%d", i);
		strcat(resul_file, numArq);
		strcat(resul_file, ".txt");
		remove(resul_file);

		strcpy(sol_file, "D:\\");
		strcat(sol_file, SOL_ROAD);
		strcat(sol_file, "_pape");
		strcat(sol_file, ".txt");
		remove(sol_file);

		strcpy(summary_file, "D:\\");
		strcat(summary_file, SUMMARY_ROAD);
		strcat(summary_file, "_pape");
		strcat(summary_file, ".txt");

		cout << "Executando o Pape." << endl;
		solRoad = designRoad(cDEsopoPape, gr, roads, resul_file, sol_file, summary_file, i);
	#endif
	}

	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	//-------------------------- ending application --------------------------------//
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	//------------------------------------------------------------------------------//
	cout << "liberando o Graph da memoria" << endl;
	freeGraph(gr);
	cout << "liberando as matrizes de distancia de estrada da memoria" << endl;

	delete []PPA;
	delete []ar;
	system("PAUSE");
	return 0;
}

